#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <termios.h>
#include <fcntl.h>
#include <string.h>
#include <sys/errno.h>

#include "lcd.h"
#include "text.h"


//////////////////////////////////////////////////////////////////////////
////////////////////// For Text-Mode Output //////////////////////////////
//////////////////////////////////////////////////////////////////////////

// TODO: Figure out why text_close() crashes!

int text_init(char *device) 
{
  if(!lcd.framebuf) 
    lcd.framebuf = malloc(lcd.wid * lcd.hgt);

  if(!lcd.framebuf)
    {
      text_close();
      return -1;
    }
// Debugging...
//  if(lcd.framebuf) printf("Frame buffer: %i\n", lcd.framebuf);

  text_clear();

  lcd.wid = 20;
  lcd.hgt = 4;
  lcd.cellwid = 5;
  lcd.cellhgt = 8;
  
  lcd.clear = text_clear;
  lcd.string = text_string;
  lcd.chr = text_chr;
  lcd.vbar = text_vbar;
  lcd.hbar = text_hbar;
  lcd.init_num = text_init_num;
  lcd.num = text_num;
  
  lcd.init = text_init;
  lcd.close = text_close;
  lcd.flush = text_flush;
  lcd.flush_box = text_flush_box;
  lcd.contrast = text_contrast;
  lcd.backlight = text_backlight;
  lcd.set_char = text_set_char;
  lcd.icon = text_icon;
  lcd.init_vbar = text_init_vbar;
  lcd.init_hbar = text_init_hbar;
  lcd.draw_frame = text_draw_frame;


  return 200;  // 200 is arbitrary.  (must be 1 or more)
}

void text_close() 
{
// Ack!  This shouldn't crash the program, but does..  Why??


//  printf("text_close()\n");

// This is the line which crashes.
//  if(lcd.framebuf) free(lcd.framebuf);

//  if(lcd.framebuf) printf("Frame buffer: %i\n", lcd.framebuf);
  lcd.framebuf = NULL;
//  printf("text_close() finished\n");
}

/////////////////////////////////////////////////////////////////
// Clears the LCD screen
//
void text_clear() 
{
//  write(fd, "\xFEX", 2);
  memset(lcd.framebuf, ' ', lcd.wid*lcd.hgt);
  
}


//////////////////////////////////////////////////////////////////
// Flushes all output to the lcd...
//
void text_flush()
{
  lcd.draw_frame(lcd.framebuf);
}


/////////////////////////////////////////////////////////////////
// Prints a string on the lcd display, at position (x,y).  The
// upper-left is (1,1), and the lower right should be (20,4).
//
void text_string(int x, int y, char string[]) 
{
  /* Removed in favor of frame buffering...
  char out[64];
  sprintf(out, "%cG%c%c%s", 254, x, y, lcd);
  write(fd, out, strlen(out));
  */


  int i;

  x -= 1;  // Convert 1-based coords to 0-based...
  y -= 1;
  
  for(i=0; string[i]; i++)
    {
      lcd.framebuf[(y*lcd.wid) + x + i] = string[i];
    }
}

/////////////////////////////////////////////////////////////////
// Prints a character on the lcd display, at position (x,y).  The
// upper-left is (1,1), and the lower right should be (20,4).
//
void text_chr(int x, int y, char c) 
{
  y--;
  x--;
  
  lcd.framebuf[(y*lcd.wid) + x] = c;
}



void text_contrast(int contrast) 
{
  printf("Contrast: %i\n", contrast);
}

void text_backlight(int on)
{
  if(on)
  {
    printf("Backlight ON\n");
  }
  else
  {
    printf("Backlight OFF\n");
  }
}

void text_init_vbar() 
{
  printf("Vertical bars.\n");
}

void text_init_hbar() 
{
  printf("Horizontal bars.\n");
}

void text_init_num() 
{
  printf("Big Numbers.\n");
}

void text_num(int x, int num) 
{
  printf("BigNum(%i, %i)\n", x, num);
}

void text_set_char(int n, char *dat)
{
  printf("Set Character %i\n", n);
}

/////////////////////////////////////////////////////////////////
// Draws a vertical bar; erases entire column onscreen.
//
void text_vbar(int x, int len) 
{
  int y;
  for(y=lcd.hgt; y > 0 && len>0; y--)
    {
      text_chr(x, y, '|');

      len -= lcd.cellhgt;
    }
  
}

/////////////////////////////////////////////////////////////////
// Draws a horizontal bar to the right.
//
void text_hbar(int x, int y, int len) 
{
  for(; x<lcd.wid && len>0; x++)
    {
      text_chr(x,y,'-');
      
      len -= lcd.cellwid;
    }
  
}


/////////////////////////////////////////////////////////////////
// Sets character 0 to an icon...
//
void text_icon(int which)
{
  printf("Char 0 set to icon %i\n", which);
}


void text_flush_box(int lft, int top, int rgt, int bot)
{
  text_flush();
  
}


void text_draw_frame(char *dat)
{
  int i, j;

  char out[LCD_MAX_WIDTH];

  if(!dat) return;

//  printf("Frame (%ix%i): \n%s\n", lcd.wid, lcd.hgt, dat);

  for(i=0; i<lcd.wid; i++)
    {
      out[i] = '-';
    }
  out[lcd.wid] = 0;
  printf("+%s+\n", out);
  
  
  for(i=0; i<lcd.hgt; i++)
    {
      for(j=0; j<lcd.wid; j++)
	{
	  out[j] = dat[j+(i*lcd.wid)];
	}
      out[lcd.wid] = 0;
      printf("|%s|\n", out);
      
    }
  
  for(i=0; i<lcd.wid; i++)
    {
      out[i] = '-';
    }
  out[lcd.wid] = 0;
  printf("+%s+\n", out);

}

